// somebody else's code for EasyPIC6 board with PIC18f4550
// GLCD fitted

// Glcd module connections
char GLCD_DataPort at PORTD;

sbit GLCD_CS1 at RB0_bit;
sbit GLCD_CS2 at RB1_bit;
sbit GLCD_RS  at RB2_bit;
sbit GLCD_RW  at RB3_bit;
sbit GLCD_EN  at RB4_bit;
sbit GLCD_RST at RB5_bit;
sbit GLCD_CS1_Direction at TRISB0_bit;
sbit GLCD_CS2_Direction at TRISB1_bit;
sbit GLCD_RS_Direction  at TRISB2_bit;
sbit GLCD_RW_Direction  at TRISB3_bit;
sbit GLCD_EN_Direction  at TRISB4_bit;
sbit GLCD_RST_Direction at TRISB5_bit;


char            text[] = "???.??";
unsigned        temp;
const unsigned short TEMP_RESOLUTION = 9;


void Display_Temperature(unsigned int temp2write) {
const unsigned short RES_SHIFT = TEMP_RESOLUTION - 8;
char temp_whole;
unsigned int temp_fraction;

    Ow_Reset(&PORTA, 5);                         // Onewire reset signal
    Ow_Write(&PORTA, 5, 0xCC);                   // Issue command SKIP_ROM
    Ow_Write(&PORTA, 5, 0x44);                   // Issue command CONVERT_T
    Delay_us(120);

    Ow_Reset(&PORTA, 5);
    Ow_Write(&PORTA, 5, 0xCC);                   // Issue command SKIP_ROM
    Ow_Write(&PORTA, 5, 0xBE);                   // Issue command READ_SCRATCHPAD

    temp =  Ow_Read(&PORTA, 5);
    temp += (Ow_Read(&PORTA, 5) << 8);

    temp2write = temp;
    //--- Format and display result on Lcd

    // Check if temperature is negative
    if (temp2write & 0x8000) {
       //text[0] = '-';
       //temp2write = ~temp2write + 1;
    }

    // Extract temp_whole
    temp_whole = temp2write >> RES_SHIFT ;

    // Convert temp_whole to characters
    if (temp_whole/100)
        text[0] = temp_whole/100  + 48;
    else
        text[0] = '0';

    text[1] = (temp_whole/10)%10 + 48;             // Extract tens digit
    text[2] =  temp_whole%10     + 48;             // Extract ones digit
    // Extract temp_fraction and convert it to unsigned int
    temp_fraction  = temp2write << (4-RES_SHIFT);
    temp_fraction &= 0x000F;
    temp_fraction *= 625;

    // Convert temp_fraction to characters
    text[4] =  temp_fraction/1000    + 48;         // Extract thousands digit
    text[5] = (temp_fraction/100)%10 + 48;         // Extract hundreds digit
}


void main() {
    ADCON1 = 0x0F;         // All AN pins as digital
    ADON_bit = 0;          // Turn off AD convertor
    CMCON  = 0x07;         // Turn off comparators
    PORTA = 0;
    TRISA = 0xEF;          // Set PORTA as input for ADC
    PORTB = 0;
    TRISB = 0;             // Set PORTB as output
    PORTD = 0;
    TRISD = 0;             // Set PORTD as output
    PORTE = 0;
    TRISE = 0;

    //ALM=1;

    Glcd_Init();                                             // Initialize GLCD
    Glcd_Fill(0);

    Glcd_Write_Text("BIENVENIDO !!!", 20, 4, 1) ;
    Delay_ms(1000);
    Glcd_Fill(0);


    while(1) {                            // do forever
        Display_Temperature(temp);

        //Display_Temperature(temp);
        Glcd_Box(76,47,115,55,1);
        Glcd_Write_Text(text, 78,6,0);          // Write string
        Delay_ms(1000);
    }

}